// -*- c++ -*-
// gtkutils.cc - Common functions for the gtk AWT peers.

/* Copyright (C) 2000  Free Software Foundation

   This file is part of libgcj.

This software is copyrighted work licensed under the terms of the
Libgcj License.  Please consult the file "LIBGCJ_LICENSE" for
details.  */

#include <gtk/gtk.h>

#include "gtkcommon.h"
