// This file was automatically generated by localedef.

package gnu.java.locale;

import java.util.ListResourceBundle;

public class LocaleInformation_oc_FR extends ListResourceBundle
{
  static final String decimalSeparator = LocaleInformation_fr_FR.decimalSeparator;
  static final String groupingSeparator = LocaleInformation_fr_FR.groupingSeparator;
  static final String numberFormat = LocaleInformation_fr_FR.numberFormat;
  static final String percentFormat = LocaleInformation_fr_FR.percentFormat;
  static final String[] weekdays = { null, "dimenge", "diluns", "dimars", "dimecres", "dij\u00F3us", "divendres", "disabte" };

  static final String[] shortWeekdays = { null, "dim", "lun", "mar", "mec", "j\u00F3u", "ven", "sab" };

  static final String[] shortMonths = { "gen", "feb", "mar", "abr", "mai", "jun", "jul", "ag\u00F3", "set", "oct", "nov", "dec", null };

  static final String[] months = { "geni\u00E8r", "febri\u00E8r", "mar\u00E7", "abrial", "mai", "junh", "julhet", "ag\u00F3st", "setembre", "octobre", "novembre", "decembre", null };

  static final String[] ampms = { "", "" };

  static final String shortDateFormat = "dd.MM.yyyy";
  static final String defaultTimeFormat = "";
  static final String currencySymbol = LocaleInformation_fr_FR.currencySymbol;
  static final String intlCurrencySymbol = LocaleInformation_fr_FR.intlCurrencySymbol;
  static final String currencyFormat = LocaleInformation_fr_FR.currencyFormat;

  private static final Object[][] contents =
  {
    { "weekdays", weekdays },
    { "shortWeekdays", shortWeekdays },
    { "shortMonths", shortMonths },
    { "months", months },
    { "ampms", ampms },
    { "shortDateFormat", shortDateFormat },
    { "defaultTimeFormat", defaultTimeFormat },
    { "currencySymbol", currencySymbol },
    { "intlCurrencySymbol", intlCurrencySymbol },
    { "currencyFormat", currencyFormat },
    { "decimalSeparator", decimalSeparator },
    { "groupingSeparator", groupingSeparator },
    { "numberFormat", numberFormat },
    { "percentFormat", percentFormat },
  };

  public Object[][] getContents () { return contents; }
}
