// This file was automatically generated by localedef.

package gnu.java.locale;

import java.util.ListResourceBundle;

public class LocaleInformation_mi_NZ extends ListResourceBundle
{
  static final String decimalSeparator = LocaleInformation_en_NZ.decimalSeparator;
  static final String groupingSeparator = LocaleInformation_en_NZ.groupingSeparator;
  static final String numberFormat = LocaleInformation_en_NZ.numberFormat;
  static final String percentFormat = LocaleInformation_en_NZ.percentFormat;
  static final String[] weekdays = { null, "R\u0101tapu", "Mane", "T\u016Brei", "Wenerei", "T\u0101ite", "Paraire", "H\u0101tarei" };

  static final String[] shortWeekdays = { null, "Ta", "Ma", "T\u016B", "We", "T\u0101i", "Pa", "H\u0101" };

  static final String[] shortMonths = { "Kohi", "Hui", "Pou", "Pae", "Hara", "Pipi", "H\u014Dngoi", "Here", "Mahu", "Whi-nu", "Whi-ra", "Haki", null };

  static final String[] months = { "Kohi-t\u0101tea", "Hui-tanguru", "Pout\u016B-te-rangi", "Paenga-wh\u0101wh\u0101", "Haratua", "Pipiri", "H\u014Dngoingoi", "Here-turi-k\u014Dk\u0101", "Mahuru", "Whiringa-\u0101-nuku", "Whiringa-\u0101-rangi", "Hakihea", null };

  static final String[] ampms = { "", "" };

  static final String shortDateFormat = "dd/MM/yy";
  static final String defaultTimeFormat = "";
  static final String currencySymbol = LocaleInformation_en_NZ.currencySymbol;
  static final String intlCurrencySymbol = LocaleInformation_en_NZ.intlCurrencySymbol;
  static final String currencyFormat = LocaleInformation_en_NZ.currencyFormat;

  private static final Object[][] contents =
  {
    { "weekdays", weekdays },
    { "shortWeekdays", shortWeekdays },
    { "shortMonths", shortMonths },
    { "months", months },
    { "ampms", ampms },
    { "shortDateFormat", shortDateFormat },
    { "defaultTimeFormat", defaultTimeFormat },
    { "currencySymbol", currencySymbol },
    { "intlCurrencySymbol", intlCurrencySymbol },
    { "currencyFormat", currencyFormat },
    { "decimalSeparator", decimalSeparator },
    { "groupingSeparator", groupingSeparator },
    { "numberFormat", numberFormat },
    { "percentFormat", percentFormat },
  };

  public Object[][] getContents () { return contents; }
}
