/* TreeSelectionModel.java --
   Copyright (C) 2002 Free Software Foundation, Inc.

This file is part of GNU Classpath.

GNU Classpath is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU Classpath is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU Classpath; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version. */

package javax.swing.tree;

// Imports
import java.beans.PropertyChangeListener;
import javax.swing.event.TreeSelectionListener;

/**
 * TreeSelectionModel interface
 * @author Andrew Selkirk
 */
public interface TreeSelectionModel {

	public static final int SINGLE_TREE_SELECTION			= 1;
	public static final int CONTIGUOUS_TREE_SELECTION		= 2;
	public static final int DISCONTIGUOUS_TREE_SELECTION	= 4;

	public void setSelectionMode(int mode);
	public int getSelectionMode();
	public void setSelectionPath(TreePath path);
	public void setSelectionPaths(TreePath[] paths);
	public void addSelectionPath(TreePath path);
	public void addSelectionPaths(TreePath[] paths);
	public void removeSelectionPath(TreePath path);
	public void removeSelectionPaths(TreePath[] paths);
	public TreePath getSelectionPath();
	public TreePath[] getSelectionPaths();
	public int getSelectionCount();
	public boolean isPathSelected(TreePath path);
	public boolean isSelectionEmpty();
	public void clearSelection();
	public void setRowMapper(RowMapper newMapper);
	public RowMapper getRowMapper();
	public int[] getSelectionRows();
	public int getMinSelectionRow();
	public int getMaxSelectionRow();
	public boolean isRowSelected(int row);
	public void resetRowSelection();
	public int getLeadSelectionRow();
	public TreePath getLeadSelectionPath();
	public void addPropertyChangeListener(PropertyChangeListener listener);
	public void removePropertyChangeListener(PropertyChangeListener listener);
	public void addTreeSelectionListener(TreeSelectionListener x);
	public void removeTreeSelectionListener(TreeSelectionListener x);


} // TreeSelectionModel
