class f {
    final static char c = 3;
    final static byte b = 0.1;
    final static short s = 3.5f;
    final static int i = 4L;
    final static long x = 2.5;
    final static float y = 2.6;
    final static String S = new Object (); 
    void foo () {
      final int i = 2.1;
    }
}
