// This file was automatically generated by localedef.

package gnu.java.locale;

import java.util.ListResourceBundle;

public class LocaleInformation_eu_ES extends ListResourceBundle
{
  static final String decimalSeparator = ",";
  static final String groupingSeparator = "";
  static final String numberFormat = "#.###";
  static final String percentFormat = "#%";
  static final String[] weekdays = { null, "igandea", "astelehena", "asteartea", "asteazkena", "osteguna", "ostirala", "larunbata" };

  static final String[] shortWeekdays = { null, "iga", "leh", "art", "azk", "teg", "tir", "lar" };

  static final String[] shortMonths = { "urt", "ots", "mar", "api", "mai", "eka", "uzt", "abu", "ira", "urr", "aza", "abe", null };

  static final String[] months = { "urtarrila", "otsaila", "martxoa", "apirila", "maiatza", "ekaina", "uztaila", "abuztua", "iraila", "urria", "azaroa", "abendua", null };

  static final String[] ampms = { "", "" };

  static final String shortDateFormat = "yy-MM-dd";
  static final String defaultTimeFormat = "";
  static final String currencySymbol = "Pts";
  static final String intlCurrencySymbol = "ESP";
  static final String currencyFormat = "$ #,###,##0.;-$ #,###,##0.";

  private static final Object[][] contents =
  {
    { "weekdays", weekdays },
    { "shortWeekdays", shortWeekdays },
    { "shortMonths", shortMonths },
    { "months", months },
    { "ampms", ampms },
    { "shortDateFormat", shortDateFormat },
    { "defaultTimeFormat", defaultTimeFormat },
    { "currencySymbol", currencySymbol },
    { "intlCurrencySymbol", intlCurrencySymbol },
    { "currencyFormat", currencyFormat },
    { "decimalSeparator", decimalSeparator },
    { "groupingSeparator", groupingSeparator },
    { "numberFormat", numberFormat },
    { "percentFormat", percentFormat },
  };

  public Object[][] getContents () { return contents; }
}
