package javax.swing.plaf;

import java.awt.*;
import javax.swing.*;

public class TabbedPaneUI extends ComponentUI
{
    public Rectangle getTabBounds(JTabbedPane pane, int index)
    {
	return null;
    }

    public int getTabRunCount(JTabbedPane pane)
    {
	return 0;
    }

    public int tabForCoordinate(JTabbedPane pane, int x, int y)
    {
	return 0;
    }
}
                       
