class test_long {

  public static void main (String[] arg)
  {
    long a, b, c, d, e;
    e = 234L;
    d = 0x8888888888888888L;
    a = 0x0123456789ABCDEFL;
    b = 0xFEDCBA9876543210L;
    c = 0xF096A5B4C3B2E187L;
    System.out.println (a+" "+b+" "+c+" "+d+" "+e);
  }
}
