// This file was automatically generated by localedef.

package gnu.java.locale;

import java.util.ListResourceBundle;

public class LocaleInformation_gd_GB extends ListResourceBundle
{
  static final String decimalSeparator = LocaleInformation_en_GB.decimalSeparator;
  static final String groupingSeparator = LocaleInformation_en_GB.groupingSeparator;
  static final String numberFormat = LocaleInformation_en_GB.numberFormat;
  static final String percentFormat = LocaleInformation_en_GB.percentFormat;
  static final String[] weekdays = { null, "Did\u00F2mhnaich", "Diluain", "Dim\u00E0irt", "Diciadain", "Diardaoin", "Dihaoine", "Disathairne" };

  static final String[] shortWeekdays = { null, "Dido", "Dilu", "Dim", "Dic", "Diar", "Diha", "Disa" };

  static final String[] shortMonths = { "Fao", "Gea", "M\u00E0r", "Gib", "Mh\u00E0", "OgM", "luc", "L\u00F9n", "Sul", "Dam", "Sam", "D\u00F9b", null };

  static final String[] months = { "Am Faoilteach", "An Gearran", "Am M\u00E0rt", "An Giblean", "A' Mh\u00E0igh", "An t-Mh\u00ECos", "An t-luchar", "An L\u00F9nasdal", "An t-Sultain", "An Damhair", "An t-Samhain", "An D\u00F9bhlachd", null };

  static final String[] ampms = { "", "" };

  static final String shortDateFormat = "dd/MM/yy";
  static final String defaultTimeFormat = "";
  static final String currencySymbol = LocaleInformation_en_GB.currencySymbol;
  static final String intlCurrencySymbol = LocaleInformation_en_GB.intlCurrencySymbol;
  static final String currencyFormat = LocaleInformation_en_GB.currencyFormat;

  private static final Object[][] contents =
  {
    { "weekdays", weekdays },
    { "shortWeekdays", shortWeekdays },
    { "shortMonths", shortMonths },
    { "months", months },
    { "ampms", ampms },
    { "shortDateFormat", shortDateFormat },
    { "defaultTimeFormat", defaultTimeFormat },
    { "currencySymbol", currencySymbol },
    { "intlCurrencySymbol", intlCurrencySymbol },
    { "currencyFormat", currencyFormat },
    { "decimalSeparator", decimalSeparator },
    { "groupingSeparator", groupingSeparator },
    { "numberFormat", numberFormat },
    { "percentFormat", percentFormat },
  };

  public Object[][] getContents () { return contents; }
}
