// This file was automatically generated by localedef.

package gnu.java.locale;

import java.util.ListResourceBundle;

public class LocaleInformation_kl_GL extends ListResourceBundle
{
  static final String decimalSeparator = LocaleInformation_da_DK.decimalSeparator;
  static final String groupingSeparator = LocaleInformation_da_DK.groupingSeparator;
  static final String numberFormat = LocaleInformation_da_DK.numberFormat;
  static final String percentFormat = LocaleInformation_da_DK.percentFormat;
  static final String[] weekdays = { null, "sabaat", "ataasinngorneq", "marlunngorneq", "pingasunngorneq", "sisamanngorneq", "tallimanngorneq", "arfininngorneq" };

  static final String[] shortWeekdays = { null, "sab", "ata", "mar", "pin", "sis", "tal", "arf" };

  static final String[] shortMonths = { "jan", "feb", "mar", "apr", "maj", "jun", "jul", "aug", "sep", "okt", "nov", "dec", null };

  static final String[] months = { "januari", "februari", "martsi", "aprili", "maji", "juni", "juli", "augustusi", "septemberi", "oktoberi", "novemberi", "decemberi", null };

  static final String[] ampms = { "", "" };

  static final String shortDateFormat = "dd MMM yyyy";
  static final String defaultTimeFormat = "";
  static final String currencySymbol = LocaleInformation_da_DK.currencySymbol;
  static final String intlCurrencySymbol = LocaleInformation_da_DK.intlCurrencySymbol;
  static final String currencyFormat = LocaleInformation_da_DK.currencyFormat;

  private static final Object[][] contents =
  {
    { "weekdays", weekdays },
    { "shortWeekdays", shortWeekdays },
    { "shortMonths", shortMonths },
    { "months", months },
    { "ampms", ampms },
    { "shortDateFormat", shortDateFormat },
    { "defaultTimeFormat", defaultTimeFormat },
    { "currencySymbol", currencySymbol },
    { "intlCurrencySymbol", intlCurrencySymbol },
    { "currencyFormat", currencyFormat },
    { "decimalSeparator", decimalSeparator },
    { "groupingSeparator", groupingSeparator },
    { "numberFormat", numberFormat },
    { "percentFormat", percentFormat },
  };

  public Object[][] getContents () { return contents; }
}
