import support.PR129_A;

public class PR129_B { 
  public static void main ( String[] args ) { 
    int length = PR129_A.strArr.length; 
    System.out.println ( "Array length: " + length ); 
  } 
} 

