
/**
 * assignment_2.java
 *
 *
 * Created: Thu Apr 13 16:48:40 2000
 *
 * @author Oliver Brandt
 * @version
 */

public class assignment_2  {
    public assignment_2 () 
    {
    }
    
    public int foo ()
    {
        int result;
        
        try
        {
            result = 0;
        }
        finally
        {
        }
        return result;
        
    }
    
}// test
