package javax.swing.plaf.basic;
import java.awt.Color;
/**
 * STUBBED
 */
public class BasicBorders
{
  public static class ButtonBorder
  {
  } // class ButtonBorder
  public static class FieldBorder
  {
    public FieldBorder(Color shadow, Color darkShadow,
                       Color highlight, Color lightHighlight)
    {
    }
  } // class FieldBorder
  public static class MarginBorder
  {
  } // class MarginBorder
  public static class MenuBarBorder
  {
    public MenuBarBorder(Color shadow, Color highlight)
    {
    }
  } // class MenuBarBorder
  public static class RadioButtonBorder
  {
  } // class RadioButtonBorder
  public static class RolloverButtonBorder
  {
  } // class RolloverButtonBorder
  public static class SplitPaneBorder
  {
    public SplitPaneBorder(Color highlight, Color shadow)
    {
    }
  } // class SplitPaneBorder
  public static class ToggleButtonBorder
  {
  } // class ToggleButtonBorder
} // class BasicBorders
