// This file was automatically generated by localedef.

package gnu.java.locale;

import java.util.ListResourceBundle;

public class LocaleInformation_tl_PH extends ListResourceBundle
{
  static final String decimalSeparator = LocaleInformation_en_US.decimalSeparator;
  static final String groupingSeparator = LocaleInformation_en_US.groupingSeparator;
  static final String numberFormat = LocaleInformation_en_US.numberFormat;
  static final String percentFormat = LocaleInformation_en_US.percentFormat;
  static final String[] weekdays = { null, "Linggo", "Lunes", "Martes", "Miyerkoles", "Huwebes", "Biyernes", "Sabado" };

  static final String[] shortWeekdays = { null, "Lin", "Lun", "Mar", "Miy", "Huw", "Biy", "Sab" };

  static final String[] shortMonths = { "Ene", "Peb", "Mar", "Abr", "May", "Hun", "Hul", "Ago", "Sep", "Okt", "Nob", "Dis", null };

  static final String[] months = { "Enero", "Pebrero", "Marso", "Abril", "Mayo", "Hunyo", "Hulyo", "Agosto", "Septiyembre", "Oktubre", "Nobiyembre", "Disyembre", null };

  static final String[] ampms = { "AM", "PM" };

  static final String shortDateFormat = "MM/dd/yy";
  static final String defaultTimeFormat = "hh:m:s a";
  static final String currencySymbol = "PhP";
  static final String intlCurrencySymbol = "PHP";
  static final String currencyFormat = "$#,###,##0.00;-$ #,###,##0.00";

  private static final Object[][] contents =
  {
    { "weekdays", weekdays },
    { "shortWeekdays", shortWeekdays },
    { "shortMonths", shortMonths },
    { "months", months },
    { "ampms", ampms },
    { "shortDateFormat", shortDateFormat },
    { "defaultTimeFormat", defaultTimeFormat },
    { "currencySymbol", currencySymbol },
    { "intlCurrencySymbol", intlCurrencySymbol },
    { "currencyFormat", currencyFormat },
    { "decimalSeparator", decimalSeparator },
    { "groupingSeparator", groupingSeparator },
    { "numberFormat", numberFormat },
    { "percentFormat", percentFormat },
  };

  public Object[][] getContents () { return contents; }
}
