// This file was automatically generated by localedef.

package gnu.java.locale;

import java.util.ListResourceBundle;

public class LocaleInformation_mt_MT extends ListResourceBundle
{
  static final String decimalSeparator = ".";
  static final String groupingSeparator = ",";
  static final String numberFormat = "#,##0.###";
  static final String percentFormat = "#,##0%";
  static final String[] weekdays = { null, "il-\u0126add", "it-Tnejn", "it-Tlieta", "l-Erbg\u0127a", "il-\u0126amis", "il-\u0120img\u0127a", "is-Sibt" };

  static final String[] shortWeekdays = { null, "\u0126ad", "Tne", "Tli", "Erb", "\u0126am", "\u0120im", "Sib" };

  static final String[] shortMonths = { "Jan", "Fra", "Mar", "Apr", "Mej", "\u0120un", "Lul", "Awi", "Set", "Ott", "Nov", "Di\u010B", null };

  static final String[] months = { "Jannar", "Frar", "Marzu", "April", "Mejju", "\u0120unju", "Lulju", "Awissu", "Settembru", "Ottubru", "Novembru", "Di\u010Bembru ", null };

  static final String[] ampms = { "", "" };

  static final String shortDateFormat = "EEEE, dd ta MMM, yyyy";
  static final String defaultTimeFormat = "hh:m:s a z";
  static final String currencySymbol = "Lm";
  static final String intlCurrencySymbol = "MTL";
  static final String currencyFormat = "$#,##0.00;($#,##0.00)";

  private static final Object[][] contents =
  {
    { "weekdays", weekdays },
    { "shortWeekdays", shortWeekdays },
    { "shortMonths", shortMonths },
    { "months", months },
    { "ampms", ampms },
    { "shortDateFormat", shortDateFormat },
    { "defaultTimeFormat", defaultTimeFormat },
    { "currencySymbol", currencySymbol },
    { "intlCurrencySymbol", intlCurrencySymbol },
    { "currencyFormat", currencyFormat },
    { "decimalSeparator", decimalSeparator },
    { "groupingSeparator", groupingSeparator },
    { "numberFormat", numberFormat },
    { "percentFormat", percentFormat },
  };

  public Object[][] getContents () { return contents; }
}
