package javax.swing.plaf;
import java.awt.Component;
import java.awt.Graphics;
import java.io.Serializable;
import javax.swing.Icon;
/**
 * STUBBED
 */
public class IconUIResource implements Icon, UIResource, Serializable
{
  public IconUIResource(Icon delegate)
  {
  }
  public void paintIcon(Component c, Graphics g, int x, int y)
  {
  }
  public int getIconWidth()
  {
    return 0;
  }
  public int getIconHeight()
  {
    return 0;
  }
} // class IconUIResource
